# coding: utf-8

import uuid
from flask import Blueprint, g, redirect, render_template, request, session
from flask import url_for, flash

from SWSCloudCore.controllers.billing import ControllerBilling
from SWSCloudCore.controllers.common import ControllerCommon
from SWSCloudCore.controllers.common import ControllerMessagesEmail
from SWSCloudCore.controllers.containers import ControllerContainers
from SWSCloudCore.controllers.containers import ControllerContainersStatistics
from SWSCloudCore.controllers.containers import ControllerContainersStatisticsState
from SWSCloudCore.controllers.datacenters import ControllerDataCenters
from SWSCloudCore.controllers.ips import ControllerIps
from SWSCloudCore.controllers.tasks import ControllerTasks
from SWSCloudCore.controllers.users import ControllerSSHKey
from SWSCloudCore.controllers.users import ControllerUsers
from SWSCloudWeb.common import requires_login

viewContainers = Blueprint('containers', __name__, url_prefix='/compute/containers')


@viewContainers.route('/')
@requires_login
def index():
    return render_template(
        'default/compute/containers/index.html',
        # get containers list
        containers=ControllerContainers(session['user_id']).get_items())


@viewContainers.route('/create', methods=['GET', 'POST'])
@requires_login
def create():
    user_balance = ControllerBilling().get(session['user_id'])
    user_ssh = ControllerSSHKey(session['user_id'])

    if request.method == "POST":
        # check user money
        if user_balance <= 0:
            flash(u'Недостаточно средств на аккаунте')
            return redirect(url_for('containers.create'))

        # select server from selected region with available ip-addresses
        # select IP
        select_ip = ControllerIps().getfree(request.form['datacenter'])
        # mark ip as busy (taken)
        ControllerIps().setbusy(select_ip.id)
        # generate password for container user
        password = ControllerCommon().generate_password(size=14)

        new_container = {
            'container_id': str(uuid.uuid4()),
            'datacenter_id': str(select_ip.datacenter.id),
            'server_id': str(select_ip.server.id),
            'ipv4': select_ip.ipv4,
            'ipv6': select_ip.ipv6,
            'ipv4_gateway': select_ip.ipv4_gateway,
            'ipv6_gateway': select_ip.ipv6_gateway,
            'username': 'ubuntu',
            'password': password,
            'ssh_key': '',
        }

        # sshkey
        if user_ssh.check():
            new_container['ssh_key'] = user_ssh.get()

        # create container record in database
        # status 4: creation
        status = 4
        container_create = ControllerContainers(session['user_id']).create(
            new_container['container_id'],
            new_container['datacenter_id'],
            new_container['server_id'],
            new_container['ipv4'],
            new_container['ipv6'],
            status
        )
        # create default state data
        ControllerContainersStatisticsState().set(new_container['container_id'], dict())
        if container_create:
            # create task for create new container
            ControllerTasks(session['user_id']).create(
                new_container['datacenter_id'],
                new_container['server_id'],
                'container_create',
                0,
                container_id=new_container['container_id'],
                ipv4=new_container['ipv4'],
                ipv6=new_container['ipv6'],
                ipv4_gateway=new_container['ipv4_gateway'],
                ipv6_gateway=new_container['ipv6_gateway'],
                username=new_container['username'],
                password=new_container['password'],
                ssh_key=new_container['ssh_key']
            )

            # send mail message with recovery code
            message_parts = []

            if new_container['ipv4']:
                message_parts.append(u"IPv4: %s" % new_container['ipv4'])
            if new_container['ipv6']:
                message_parts.append(u"IPv6: %s" % new_container['ipv6'])
            message_parts.append(u"Пользователь: %s" % new_container['username'])
            message_parts.append(u"Пароль: %s" % new_container['password'])
            if new_container['ssh_key']:
                message_parts.append(u"SSH ключ: добавлен")

            message = '<br/>\n'.join(message_parts)
            subject = u'GoCloud.ru: Новый контейнер'
            lead = u"""Поздравляем с новым контейнером."""
            callout = u"""
            Для входа в личный кабинет воспользуйтесь
            <a href="https://gocloud.ru/account/login">страницей авторизации</a>.
            """

            user_data = ControllerUsers(session['user_id']).get()

            email = ControllerMessagesEmail()
            email.send(title=subject, to=user_data.email, lead=lead, message=message, callout=callout)

            return redirect(url_for('containers.index'))
        else:
            # mark ip as free
            ControllerIps().setfree(select_ip.id)
    # Get datacenters list
    datacenters = ControllerDataCenters().get()
    #
    return render_template(
        'default/compute/containers/create.html',
        datacenters=datacenters
    )


@viewContainers.route('/delete/<uuid:container_id>.html', methods=['GET', 'POST'])
@requires_login
def delete(container_id):
    #
    containers = ControllerContainers(session['user_id'])
    # check the user have a selected rule
    # if user not have a container then redirect to the container list
    if not containers.check_exists_item(container_id):
        return redirect(url_for('containers.index'))
    # get container details
    container_details = ControllerContainers(session['user_id']).get_item(container_id)
    # POST
    if request.method == "POST":
        # Обновляем статус "5" для правила
        containers.set_status(container_id, 5)
        # Создание задания
        ControllerTasks(session['user_id']).create(
            container_details.datacenter.id,
            container_details.server.id,
            'container_delete',
            0,
            container_id=container_id
        )
        # TODO: send email container was deleted about
        # Редиректим на страницу со всеми правилами
        flash(u'Контейнер был удалён')
        return redirect(url_for('containers.index'))
    return render_template(
            'default/compute/containers/delete.html',
            container=container_details
    )


@viewContainers.route('/settings/<uuid:container_id>', methods=['GET', 'POST'])
@requires_login
def settings(container_id):
    containers = ControllerContainers(session['user_id'])

    # check the user have a selected rule
    # if user not have a container then redirect to the container list
    if not containers.check_exists_item(container_id):
        return redirect(url_for('containers.index'))

    # get container details
    container_details = containers.get_item(container_id)

    if request.method == 'POST':
        if request.form['action'] == 'set_status':
            if request.form['status'] == 'inactive':
                containers.set_status(container_id, 3)
                # Создание задания
                ControllerTasks(session['user_id']).create(
                    container_details.datacenter.id,
                    container_details.server.id,
                    'container_stop',
                    0,
                    container_id=container_id
                )
                return redirect(url_for('containers.settings', container_id=container_id))

            if request.form['status'] == 'active':
                balance = ControllerBilling().get(session['user_id'])

                if balance <= 0:
                    flash(u'Недостаточно средств на балансе.')
                    return redirect(url_for('containers.settings', container_id=container_id))

                containers.set_status(container_id, 2)
                # Создание задания
                ControllerTasks(session['user_id']).create(
                    container_details.datacenter.id,
                    container_details.server.id,
                    'container_start',
                    0,
                    container_id=container_details.id
                )
                return redirect(url_for('containers.settings', container_id=container_id))

    return render_template(
        'default/compute/containers/settings.html',
        container=container_details
    )


@viewContainers.route('/stats/<uuid:container_id>.html')
@requires_login
def stats(container_id):
    # init
    containers = ControllerContainers(session['user_id'])
    # check the user have a selected rule
    # if user not have a container then redirect to the container list
    if not containers.check_exists_item(container_id):
        return redirect(url_for('containers.index'))
    # get container details
    container_details = containers.get_item(container_id)
    # print ControllerContainersStatisticsState().get(container_id)

    statistics = []
    for s in ControllerContainersStatistics(container_id).size_get(1):
        # print time.strftime(s.created, '%Y')
        # print datetime.datetime.strptime(s.created, '%Y')
        # print time.strftime("%B %d %Y", s.created)
        created = s.created
        statistics.append({
            'year': created.strftime('%Y'),
            'month': created.strftime('%m'),
            'day': created.strftime('%d'),
            'hour': created.strftime('%H'),
            'minute': created.strftime('%M'),
            'data': s.memory
        })
    # return
    return render_template(
            'default/compute/containers/stats.html',
            container=container_details,
            stats_memory=statistics)
