# coding: utf-8

from setuptools import setup

setup(
    name='SWSCloudCore',
    version='2.7.9',
    author='Vyacheslav Anzhiganov',
    author_email='hello@anzhiganov.com',
    packages=[
        # API
        'SWSCloudAPI',
        'SWSCloudAPI.API',
        'SWSCloudServerAPI',
        'SWSCloudServerAPI.ServerAPI',
        # Core
        'SWSCloudCore',
        'SWSCloudCore.controllers',
        'SWSCloudCore.controllers.administrators',
        'SWSCloudCore.controllers.billing',
        'SWSCloudCore.controllers.common',
        'SWSCloudCore.controllers.containers',
        'SWSCloudCore.controllers.datacenters',
        'SWSCloudCore.controllers.ips',
        'SWSCloudCore.controllers.kb',
        'SWSCloudCore.controllers.payments',
        'SWSCloudCore.controllers.plans',
        'SWSCloudCore.controllers.servers',
        'SWSCloudCore.controllers.settings',
        'SWSCloudCore.controllers.tasks',
        'SWSCloudCore.controllers.teams',
        'SWSCloudCore.controllers.user_settings',
        'SWSCloudCore.controllers.users',
        'SWSCloudCore.controllers.vms',
        # Web
        'SWSCloudWeb',
        'SWSCloudWeb.views',
        'SWSCloudWeb.views.account',
        'SWSCloudWeb.views.bills',
        'SWSCloudWeb.views.compute',
        'SWSCloudWeb.views.compute.containers',
        'SWSCloudWeb.views.compute.vms',
        'SWSCloudWeb.views.payments',
        'SWSCloudWeb.views.settings',
        'SWSCloudWeb.views.support',
        'SWSCloudWeb.views.tasks',
        # Administrator
        'SWSCloudAdministrator',
        'SWSCloudAdministrator.Administrator',
        'SWSCloudAdministrator.Administrator.compute',
        # StatisticsClient
        'SWSStatisticsClient',
    ],
    package_data={
        'SWSCloudWeb': [
            # Static files
            'static/css/*.css',
            'static/images/*.png',
            'static/images/cloudns/*.png',
            'static/images/control/*.png',
            'static/images/payment/*.png',
            'static/images/social/*.png',
            'static/images/promo/*.png',
            'static/js/*.js',
            'static/js/foundation/*.js',
            'static/js/vendor/*.js',
            # New homepage
            'static/assets/css/*.css',
            'static/assets/img/*.png',
            'static/assets/img/testimonials/*.png',
            'static/assets/img/testimonials/*.jpg',
            'static/assets/js/*.js',
            # Templates
            'templates/*.html',
            'templates/default/*.html',
            'templates/default/billing/*.html',
            'templates/default/compute/containers/*.html',
            'templates/default/compute/vms/*.html',
            'templates/default/documents/*.html',
            'templates/default/homepage/*.html',
            'templates/default/id/*.html',
            'templates/default/kb/*.html',
            'templates/default/payment/*.html',
            'templates/default/payment/robokassa/*.html',
            'templates/default/support/*.html',
            'templates/default/tasks/*.html',
            'templates/email/simple/*.html',
            'templates/email/simple/*.css',
            # GoCloud2016
            'templates/gocloud2016/layouts/*.html',
            'templates/gocloud2016/macros/*.html',
            'templates/gocloud2016/pages/*.html',
            # Errors
            'templates/errors/*.html',
        ],
        'SWSCloudAdministrator': [
            # Static files
            'static/css/*.css',
            'static/images/*.png',
            'static/images/cloudns/*.png',
            'static/images/control/*.png',
            'static/images/payment/*.png',
            'static/images/social/*.png',
            'static/images/promo/*.png',
            'static/js/*.js',
            'static/js/foundation/*.js',
            'static/js/vendor/*.js',
            #
            'templates/administrator/*.html',
            'templates/administrator/compute/containers/*.html',
            'templates/administrator/compute/vms/*.html',
            'templates/administrator/datacenters/*.html',
            'templates/administrator/email/*.html',
            'templates/administrator/ips/*.html',
            'templates/administrator/plans/*.html',
            'templates/administrator/servers/*.html',
            'templates/administrator/settings/*.html',
            'templates/administrator/settings/messages/*.html',
            'templates/administrator/tasks/*.html',
            'templates/administrator/users/*.html',
            # 'templates/administrator/payments/*.html',
            'templates/administrator/wiki/*.html',
            'templates/administrator/wiki/article/*.html',
            'templates/administrator/wiki/category/*.html',
            # Errors
            'templates/errors/*.html',
        ],
    },
    scripts=[
        'cloud-db-init.py',
        'cloud-settings.py',
        'cloud-settings-init.py',
        'cloud-runserver.py',
        'cloud-runserver-admin.py',
        # accounts
        'cloud-admin-add.py',
        # 'swscloud-admin-delete.py',
        'cloud-admin-ls.py',
        'cloud-admin-password.py',
        # datacenters
        'cloud-dc-add.py',
        'cloud-dc-ls.py',
        # servers
        'cloud-server-add.py',
        'cloud-server-ls.py',
        # billing
        'cloud-cron-balance.py',
    ],
    install_requires=[
        'Flask==0.10',
        'Flask-Markdown==0.3',
        'Flask-Babel==0.9',
        'flask-peewee==0.6.0',
        'flask-multistatic',
        'Jinja2==2.8',
        'peewee==2.8',
        'validators==0.10',
        'psycopg2==2.6.1',
        'configparser',
        'flask_httpauth==3.1.1',
        'requests==2.7',
        'uWSGI==2.0.11.1',
        'wsgiref==0.1.2',
        'sshpubkeys==1.0.6',
        'dateutils',
    ]
)
